#!/usr/bin/perl
#
# add2muwed.pl - add mp3 files to your database
#
# Copyright (c) by Andreas S. Krebs (andreas.krebs@web.de)
#
# this script tries to get the information for the muwed from the mp3 file in the following way:
# 1. It tries to split the filename into "ARTIST - SONG.mp3"
# 2. It tries to read the ID3 tags from the mp3 file
# 3. If the artist and song title constructed from the filename is longer than that from the ID3 tag it's used 
# 4. It tries to read the mp3 header from the mp3 file
#
# NOTE:
# this script expects the filename given to it relative to your mp3 root dir, otherwise the database entries
# will be incorrect
#

use Pg;
use MP3::Tag;
use MP3::Info;
use Getopt::Std;

$DBNAME="muwed";
$DBUSER="postgres";

$USE_ALBUMTAG=1;
$USE_NOTETAG=0;
$USE_FILENAME=1;

$USAGE="Usage: add2muwed.pl [-u] file ...\n  -u: update values for existing files\n";

getopts('u') or die $USAGE;

if($#ARGV < 0)
{
  print "$USAGE";
  exit 0;
}

# try to connect to the database
$conn=Pg::connectdb("dbname=$DBNAME user=$DBUSER");
( PGRES_CONNECTION_OK eq $conn->status )
    and print "Pg::connectdb ........... ok\n"
    or  die   "Pg::connectdb ........... not ok: ", $conn->errorMessage;

foreach $file (@ARGV)
{
  processFile($file);
}

1;

sub processFile
{
  local($file)=shift(@_);
  $file=~s/^.\///;
  local($title, $artist, $composer, $album, $note, $year, $genre, $track, $size, $length, $br, $flags, $tagflags);
  local($inx, $line, $efile, $fartist, $ftitle);
  local(%values)=();

  $efile=$file;
  $efile=~ s/`/\\`/g;
  $line=`basename \"$efile\"`;

  print "processing: $file\n";  
  
  $inx=index($line, ".mp3");
  $artist="";
  $title="";
  if($inx > 0 && $USE_FILENAME==1)
  {
    $line=substr($line, 0, $inx);
    ($fartist, $ftitle)=split(' - ', $line, 2);
  }

  $fartist=&removeSpaces($fartist);
  $ftitle=&removeSpaces($ftitle);

  ($title, $artist, $composer, $album, $note, $year, $genre, $track, $tagflags)=&getTags($file);
  if(length($ftitle) >= length($title) && length($fartist) >= length($artist))
  {
    $title=$ftitle;
    $artist=$fartist;
  }

  if($artist ne "" && $title ne "")
  {
    ($size, $length, $br, $flags)=&getInfo($efile);

    $flags|=$tagflags;
    printf("$file: $artist - $title ($album): $size bytes, $length secs, $br kbs, 0x%.4X\n", $flags);

    $values{"title"}    =$title;
    $values{"artist"}   =$artist;
    $values{"album"}    =$album;
    $values{"note"}     =$note;
    $values{"year"}     =$year;
    $values{"genre"}    =$genre;
    $values{"filesize"} =$size;
    $values{"length"}   =$length;
    $values{"bitrate"}  =$br;
    $values{"flags"}    =$flags;
    $values{"track"}    =$track;
    $values{"composer"} =$composer;

    &update_entry($file, \%values);
  }
  else
  {
    print("IGNORING $file: missing artist and title\n");
  }
}

sub update_entry
{
  local($file)=shift(@_);
  local($valref)=shift(@_);
  local($query, $result);

  $file=&db_escape($file);
  
  # first check if file is already in database
  $query="SELECT * FROM songs WHERE file=$file";
  $result=$conn->exec($query);

  if($result->ntuples eq 0)	# was file not present in database ?
  {
    local($fields, $vals);

    $fields="file";
    $vals=$file;
    foreach(keys %$valref)
    {
      $fields.=", ".$_;
      $vals.=", ".&db_escape($$valref{$_});
    }

    $query="INSERT INTO songs ($fields) VALUES ($vals)";
    print "$query\n";

    $result=$conn->exec($query);
  }
  elsif($opt_u eq 1)		# was an update for the file requested ?
  {
    local(@sets)=();
    local($setvals);

    foreach(keys %$valref)
    {
      @sets=(@sets, $_."=".&db_escape($$valref{$_}));
    }

    $setvals=join(', ', @sets);
    $query="UPDATE songs SET $setvals WHERE file=$file";
    print "$query\n";

    $result=$conn->exec($query);
  }

  print $conn->errorMessage;
}

sub db_escape
{
  local($field)=shift(@_);

  $field=~ s/'/''/g;
#  $field=~ s/`/\\`/g;

  if($field ne "")
  {
    $field="'$field'";
  }
  else
  {
    $field="NULL";
  }

  return ($field);
}

sub getTags
{
  local($file)=shift(@_);
  local($i, $mp3, $title, $artist, $composer, $album, $note, $year, $genre, $track, $flags);

  $mp3=MP3::Tag->new($file);
  $mp3->getTags();
  $flags=0;

  @_=$mp3->autoinfo();
  for($i=0; $i<=$#_; $i++)
  {
    if($_[$i] eq "undef")
    {
      $_[$i]="";
    }
  }

  ($title, $track, $artist, $album) = @_;
  if($USE_ALBUMTAG!=1)
  {
    $album="";
  }

  if(exists $mp3->{ID3v1})
  {
    if($USE_NOTETAG==1)
    {
      $note=$mp3->{ID3v1}->comment;
    }

    $year=$mp3->{ID3v1}->year;
    $genre=$mp3->{ID3v1}->genreID;
  }
  if(exists $mp3->{ID3v2})	# do we have an ID3v2 stuff (like APIC)
  {
    $flags|=0x1000;		# we do have ID3v2 tags
    
    local($frames)=$mp3->{ID3v2}->getFrameIDs();

    if(exists $frames->{"TCOM"})
    {
      $composer=$mp3->{ID3v2}->getFrame("TCOM");
    }

    if(exists $frames->{"APIC"})
    {
      $flags|=0x2000;
    }
  }

  if($track<=0)
  {
    $track="";
  }

  return ($title, $artist, $composer, $album, $note, $year, $genre, $track, $flags);
}

sub getInfo
{
  local($file)=shift(@_);
  local($info, $size, $length, $br, $flags);

  $info=get_mp3info($file);
  $flags=0;

  $size=$info->{SIZE};
  $length=$info->{SECS};
  $length=~ s/\..*//g;

  $flags|=$info->{LAYER};
  $br=$info->{BITRATE};

  if($info->{STEREO})
  {
    $flags|=0x0004;
  }

  if($info->{MODE} eq 1)	# joint stereo ?
  {
    $flags|=0x0008;
  }

  if($info->{VBR})
  {
    $flags|=0x0010;
  }
  
  return ($size, $length, $br, $flags);
}

sub removeSpaces
{
  local($txt)=shift(@_);

  while(substr($txt, 0, 1) eq " " || substr($txt, 0, 1) eq "\t")
  {
    $txt=substr($txt, 1);
  }

  while(substr($txt, -1, 1) eq " " || substr($txt, -1, 1) eq "\t")
  {
    $txt=substr($txt, 0, -1);
  }

  $txt=~s/"/\\"/g;

  return $txt;
}

